@@ -66,8 +66,8 @@ class MetaDocument(type):
 
 
 class Document(metaclass=MetaDocument):
-    _database_ = "database"
-    _collection_ = "collection"
+    _database_: str
+    _collection_: str
     _loop_: AbstractEventLoop
     _client_: AsyncIOMotorClient
     _relations_ = {}
@@ -203,18 +203,29 @@ class Document(metaclass=MetaDocument):
             {other_field: self[self_field]},
             {"$pull": {other_field: self[self_field]}})
 
-    async def _delete_related(self):
-        """Deletes related documents or pops field values."""
+    async def _delete_other(self, doc, delete_info):
 
-        self_properties = set({v for v in self.__class__.__dict__.values()
-                               if isinstance(v, property)})
+        if delete_info["on_delete"] is DeleteDocument:
+            for doc in await doc.many(**{
+                delete_info["self_field"]: self[delete_info["other_field"]]
+            }):
+                await doc.delete()
+        elif delete_info["on_delete"] is PopFromArray:
+            await self._delete_related_pop(document=doc,
+                                           self_field=delete_info["self_field"],
+                                           other_field=delete_info["other_field"])
 
-        relations = set(Document._relations_).intersection(self_properties)
+        elif delete_info["on_delete"] is DeleteField:
+            await doc.collection.update_many(
+                {delete_info["self_field"]: self[delete_info["other_field"]]},
+                {"$unset": {delete_info["self_field"]: ""}})
+
+    async def _delete_by_prop(self, relations):
+
+        for prop, delete_info in relations.items():
 
-        for prop in relations:
-            delete_info = self._relations_[prop]
             property_name = prop.fget.__name__
-            document = delete_info["document_func"](self)
+            document = delete_info["other_document"]()
 
             if delete_info["on_delete"] is DeleteDocument:
                 for doc in await getattr(self, property_name):
@@ -230,40 +241,77 @@ class Document(metaclass=MetaDocument):
                     {delete_info["other_field"]: self[delete_info["self_field"]]},
                     {"$unset": {delete_info["other_field"]: ""}})
 
+    async def _delete_related(self):
+        """Deletes related documents or pops field values."""
+
+        self_properties = set({v for v in self.__class__.__dict__.values()
+                               if isinstance(v, property)})
+
+        other_properties = {
+            k: v for k, v in Document._relations_.items() if v["other_document"]() is
+                self.__class__}
+
+        relations = set(Document._relations_).intersection(self_properties)
+
+        await self._delete_by_prop({prop: delete_info for prop, delete_info
+            in Document._relations_.items() if prop in relations
+        })
+
+        for prop, delete_info in other_properties.copy().items():
+
+            doc = delete_info["self_document"]()
+
+            await self._delete_other(doc, delete_info)
+
     async def delete(self):
         """Delete current document and related fields or documents base on related."""
 
+        result = await self.collection.delete_one({"_id": self._id})
         await self._delete_related()
-        return await self.collection.delete_one({"_id": self._id})
+        return result
 
     @staticmethod
-    def related(self_field, other_field, multiple=True, on_delete=None):
+    def related(self_path, other_path, multiple=True, on_delete=None):
         """Decorator for related documents.
 
         :param on_delete: actions to take when document is deleted
-        :param self_field: primary key field of current document
-        :param other_field: search field of searched document
+        :param self_path: Document.key to self pk
+        :param other_path: Document.key to other pk
         :param multiple: return multiple documents or only one
         """
 
         def func_wrapper(func):
 
+            self_field = ".".join(self_path.split(".")[1:])
+            other_field = ".".join(other_path.split(".")[1:])
+
+            def get_self_document():
+                for subclass in Document.__subclasses__():
+                    if subclass.__name__ == self_path.split(".")[0]:
+                        return subclass
+
+            def get_other_document():
+                for subclass in Document.__subclasses__():
+                    if subclass.__name__ == other_path.split(".")[0]:
+                        return subclass
+
             @wraps(func)
             async def fget(self):
+                other_document = get_other_document()
+
                 if multiple:
-                    return await func(self).many(self._loop_,
-                                                 **{other_field: self[self_field]})
+                    return await other_document.many(**{other_field: self[self_field]})
                 else:
-                    return await func(self).one(self._loop_,
-                                                **{other_field: self[self_field]})
+                    return await other_document.one(**{other_field: self[self_field]})
 
             result = property(fget=fget)
 
             delete_item = {
-                "document_func": func,
                 "self_field": self_field,
                 "other_field": other_field,
                 "on_delete": on_delete,
+                "self_document": get_self_document,
+                "other_document": get_other_document
             }
 
             Document._relations_[result] = delete_item
